﻿using System;
using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.PluginInfrastructure;
using Newtonsoft.Json;

namespace HeuristicLab.DefinitionLanguage {
  public static class Parser {
    private static Dictionary<string, object> instancedObjects = new Dictionary<string, object>();
    internal static object GetInstance(string str) {
      return instancedObjects[str];
    }
    private static void Init(TypeDescription[] typeDescriptions) {
      foreach (var typedesc in typeDescriptions) {
        Type t = ApplicationManager.Manager.GetType(typedesc.Type);
        if (t != null) {
          instancedObjects.Add(typedesc.Name, Activator.CreateInstance(t));
        } else {
          Console.WriteLine($"Cannot find type '{typedesc.Type}'");
        }
      }
    }

    private static void Inject(TypeDescription[] typeDescriptions) {
      foreach (var typedesc in typeDescriptions) {
        if (typedesc.Properties != null) {
          foreach (var propdesc in typedesc.Properties) {
            propdesc.Set(instancedObjects[typedesc.Name]);
            /*
            var type = instancedObjects[typedesc.Name].GetType();
            var all = type.GetProperties().Where(x => x.Name == propdesc.Key);
            var info = all.FirstOrDefault(x => x.DeclaringType == type) ?? all.First();

            object val = null;
            if (propdesc.Value != null) {
              val = ParseString(propdesc.Value, info.PropertyType);
            }
            if (propdesc.ValueRef != null) {
              val = instancedObjects[propdesc.ValueRef];
            }
            if (val != null) {
              
              info.SetValue(instancedObjects[typedesc.Name], val);
            }
            */
          }
        }
      }
    }

    private static void Save(string outputFile) {
      ContentManager.Initialize(new PersistenceContentManager());
      foreach (var o in instancedObjects.Values) {
        IExecutable alg = o as IExecutable;
        if (alg != null) {
          IStorableContent storable = alg as IStorableContent;
          if (storable != null)
            ContentManager.Save(alg as IStorableContent, outputFile, true);
        }
      }
    }
    public static void Parse(string inputFile, string outputFile) {
      TypeDescription[] td = new TypeDescription[] {
        new TypeDescription() {
          Name = "Test",
          Type = "Test",
          Properties = new Property[] {
            new PrimitiveProperty() { Key = "Key", Value = "Value" },
            new ReferenceProperty() { Key = "Key", Value = "Test" }
          }
        }
      };

      string s = JsonConvert.SerializeObject(td, Formatting.Indented);
      Console.WriteLine(s);
      td = JsonConvert.DeserializeObject<TypeDescription[]>(s);

      /*
      string s = File.ReadAllText(inputFile);
      TypeDescription[] typeDescriptions = JsonConvert.DeserializeObject<TypeDescription[]>(s);
      Init(typeDescriptions);
      Inject(typeDescriptions);
      Save(outputFile);
      */
    }

    internal static object ParseString(string str, Type targetType) {
      if (targetType == typeof(bool)) return bool.Parse(str);
      else if (targetType == typeof(string)) return str;
      else if (targetType == typeof(short)) return short.Parse(str);
      else if (targetType == typeof(int)) return int.Parse(str);
      else if (targetType == typeof(long)) return long.Parse(str);
      else if (targetType == typeof(float)) return float.Parse(str);
      else if (targetType == typeof(double)) return double.Parse(str);
      else throw new NotSupportedException($"Type {targetType.Name} is not supported!");
    }
  }
}
